@@ -22,7 +22,7 @@
 
 #include <gio/gio.h>
 
-#include "tracker-writeback.h"
+#include "tracker-writeback-module.h"
 
 G_BEGIN_DECLS
 
@@ -43,11 +43,12 @@ struct TrackerWritebackFile {
 struct TrackerWritebackFileClass {
 	TrackerWritebackClass parent_class;
 
-	gboolean              (* update_file_metadata) (TrackerWritebackFile *writeback_file,
-	                                                GFile                *file,
-	                                                GPtrArray            *values,
-	                                                TrackerClient        *client);
-	const gchar * const * (* content_types)        (TrackerWritebackFile *writeback_file);
+	gboolean              (* update_file_metadata) (TrackerWritebackFile    *writeback_file,
+	                                                GFile                   *file,
+	                                                GPtrArray               *values,
+	                                                TrackerSparqlConnection *connection,
+	                                                GCancellable            *cancellable);
+	const gchar * const * (* content_types)        (TrackerWritebackFile    *writeback_file);
 
 };
 
